;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; First test of uC programming
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.include "2313def.inc"

.def	temp1=r16
.def	dumpreg=r17

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; interrupt vector table (DS S.19)

.org 0x000
	rjmp	MAIN	; Reset Handler 
	reti		; rjmp	EXT_INT0 ; IRQ0 Handler 
	reti		; rjmp	EXT_INT1 ; IRQ1 Handler 
	reti		; rjmp	TIM_CAPT1 ; Timer1 Capture Handler 
	reti		; rjmp	TIM_COMP1 ; Timer1 Compare Handler 
	reti		; rjmp	TIM_OVF1 ; Timer1 Overflow Handler
	reti		; rjmp	TIM0_OVF ; Timer0 Overflow Handler
	reti		; rjmp	UART_RXC ; UART RX Complete Handler
	reti		; rjmp	UART_DRE ; UDR Empty Handler
	reti		; rjmp	UART_TXC ; UART TX Complete Handler
	reti		; rjmp	ANA_COMP ; Analog Comparator Handler

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; dump the whole memory, the PORTS and all registers too RS232
DUMP:
_DUMP1:					
	sbis	USR, UDRE		; rescue dumpreg
	rjmp	_DUMP1			; wait till UDR empty
	out	UDR, dumpreg

	in	dumpreg, SREG		; rescue SREG
_DUMP2:	sbis	USR, UDRE		; wait till UDR empty
	rjmp	_DUMP2
	out	UDR, dumpreg

	in	dumpreg, SPL		; rescue Stackpointer
_DUMP3:	sbis	USR, UDRE		; wait till UDR empty
	rjmp	_DUMP3
	out	UDR, dumpreg
				
	dec	dumpreg			; rescue 2 memory locations for 	
	dec	dumpreg			; ZL, ZH
	out	SPL, dumpreg

	pop	dumpreg			; rescue [SP-1]
_DUMP4:	sbis	USR, UDRE		; wait till UDR empty
	rjmp	_DUMP4
	out	UDR, dumpreg
	pop	dumpreg			; rescue [SP]
_DUMP5:	sbis	USR, UDRE		; wait till UDR empty
	rjmp	_DUMP5
	out	UDR, dumpreg

	push	ZL		; ZL is now in [SP]
	push	ZH		; ZH is now in [SP-1]

	ldi	ZH, high(RAMEND+1)	; +1 because of predecrement
	ldi	ZL, low(RAMEND+1)
	
_DUMP6:	ld	dumpreg, -Z
_DUMP7:	sbis	USR, UDRE		; wait till UDR empty
	rjmp	_DUMP7
	out	UDR, dumpreg
	mov	dumpreg, ZL
	or	dumpreg, ZH
	brne	_DUMP6

	pop	ZH
	pop	ZL

;; lets read the EEPROM now
	in	dumpreg, EEAR
	push	dumpreg
	in	dumpreg, EEDR
	push	dumpreg
	
	ldi	dumpreg, E2END		; read the whole eeprom starting at the end
_DUMP8:	out	EEAR, dumpreg
	sbi	EECR, EERE
	in	dumpreg, EEDR
_DUMP9:	sbis	USR, UDRE		; wait till UDR empty
	rjmp	_DUMP9
	out	UDR, dumpreg
	in	dumpreg, EEAR
	subi	dumpreg,1
	brcc	_DUMP8
	
	pop	dumpreg
	out	EEDR, dumpreg
	pop	dumpreg
	out	EEAR, dumpreg
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN PROGRAMM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
MAIN:

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; init stack pointer to highest address in internal SRAM
	ldi	temp1,low(RAMEND)
	out	SPL,temp1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; RS232 Init

	sbi	UCR, TXEN
	ldi	temp1, 25	;  set Baudrate to 9600 @ 4MHz
	out	UBRR, temp1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; hardware init

	ldi	temp1, 0xaa		; set PORTB=0xaa and as output 
	out	PORTB, temp1
	ldi 	temp1, 0xff
	out 	DDRB,temp1		
	ldi	temp1, 0x00		; set PORTD as input
	out 	DDRD,temp1		

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; main programm

	ldi	dumpreg, 33
	ldi	zh, 0x11
	ldi	zl, 0x22
	ldi	temp1,0xa0
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	
	rcall	dump
MAIN_LOOP:
	rjmp 	MAIN_LOOP


.ESEG
hallo: 
.db 0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6, "Hello World!",0x0a,0x0d
